package com.incubator.common.net.netty.client;

import java.net.InetSocketAddress;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.netty.OptionConfiger;
import com.incubator.common.thread.NamedThreadFactory;

import io.netty.bootstrap.Bootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;

/**
 * 默认都是后台线程
 */
public class NettyClientConfig {

	protected final Logger logger = Log4jUtil.getLogger(getClass());

	protected final Class<? extends SocketChannel> channelClass;

	protected final EventLoopGroup ioWorkers;

	/**
	 * 连接超时毫秒
	 */
	protected long connectTimeOutMills = TimeUnit.SECONDS.toMillis(3);

	public NettyClientConfig(Class<? extends SocketChannel> channelClass, EventLoopGroup ioWorkers) {
		this.channelClass = channelClass;
		this.ioWorkers = ioWorkers;
	}

	public NettyClientConfig() {
		this.channelClass = NioSocketChannel.class;
		this.ioWorkers = new NioEventLoopGroup(0, new NamedThreadFactory("ClientConfig-ioWorkers", true));
	}

	public NettyClientConfig(int ioThreads) {
		if (ioThreads < 0) {
			ioThreads = 0;
		}
		this.channelClass = NioSocketChannel.class;
		this.ioWorkers = new NioEventLoopGroup(ioThreads, new NamedThreadFactory("ClientConfig-ioWorkers", true));
	}

	public Class<? extends SocketChannel> getChannelClass() {
		return channelClass;
	}

	public EventLoopGroup getIoWorkers() {
		return ioWorkers;
	}

	public long getConnectTimeOutMills() {
		return connectTimeOutMills;
	}

	public void setConnectTimeOutMills(long connectTimeOutMills) {
		this.connectTimeOutMills = connectTimeOutMills;
	}

	public void destory() {
		if (ioWorkers != null) {
			ioWorkers.shutdownGracefully();
		}
	}

	protected final Bootstrap booter = new Bootstrap();

	Bootstrap getBooter() {
		return booter;
	}

	protected void booterInit() {
		booter.group(ioWorkers);
		booter.channel(channelClass);
		initBooterOptions(optionConfig());
	}

	/**
	 * 初始化客户端配置
	 */
	protected void initBooterOptions(OptionConfiger configer) {
		configer.option(ChannelOption.SO_KEEPALIVE, true);
		configer.option(ChannelOption.TCP_NODELAY, true);
	}

	public OptionConfiger optionConfig() {
		return new OptionConfiger() {
			@Override
			public <T> OptionConfiger option(ChannelOption<T> option, T value) {
				booter.option(option, value);
				return this;
			}
		};
	}

	/**
	 * 因为每次连接执行都会init都会被remove,所以每次调用booter都会用新的handler来进行连接配置
	 * 
	 * @param address
	 * @param init
	 * @return
	 */
	protected ChannelFuture doBooterConnect(InetSocketAddress address, final ChannelHandler init) {
		ChannelFuture cf;
		synchronized (booter) {
			if (booter.config().group() == null) {
				booterInit();
			}
			final CountDownLatch latch = new CountDownLatch(1);
			ChannelHandler handler = new ChannelInitializer<Channel>() {
				@Override
				protected void initChannel(Channel ch) throws Exception {
					ch.pipeline().addLast(init);
				}
			};

			booter.handler(handler);
			cf = booter.connect(address);
			cf.addListener(new ChannelFutureListener() {
				@Override
				public void operationComplete(ChannelFuture future) throws Exception {
					logger.trace("connect operationComplete:isDone=" + future.isDone() + ",isSuccess=" + future.isSuccess());
					if (future.isDone() && future.isSuccess()) {
						latch.countDown();
					}
				}
			});
			try {
				latch.await(getConnectTimeOutMills(), TimeUnit.MILLISECONDS);
			} catch (Exception e) {
				logger.error(e.getMessage(), e);
			}
		}
		return cf;
	}
}
