package com.incubator.common.net.netty.listener;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.Connection;
import com.incubator.common.net.ConnectionHeartbeatListener;
import io.netty.handler.codec.http.websocketx.BinaryWebSocketFrame;
import io.netty.handler.codec.http.websocketx.TextWebSocketFrame;
import org.slf4j.Logger;

import java.util.concurrent.TimeUnit;

/**
 * 具有心跳监测机制的链路监听器 如果你想使用心跳:
 * 
 * @code @author johnc
 *
 */
public abstract class ConnectionHeartbeatListenerFactory<T> implements ConnectionHeartbeatListener<Object> {

	protected final Logger logger = Log4jUtil.getLogger(getClass());

	/** 心跳验证间隔(仅当空闲时) **/
	public final static long HeartIntervalMills = TimeUnit.SECONDS.toMillis(5);

	/** 心跳开关 **/
	private boolean heartEnable;

	/** 最后异常读消息时间 **/
	public static final String LastReadTimeMills = "LastReadTimeMills";

	/** 心跳配置 **/
	public static final String HeartConfig = "HeartConfig";

	public boolean isHeartEnable() {
		return heartEnable;
	}

	public void setHeartEnable(boolean heartEnable) {
		this.heartEnable = heartEnable;
	}

	/** 心跳配置类 **/
	public static class HeartConfig {

		/** 最大发送序号(最多发送几次心跳验证) **/
		public static final int DefaultCloseMaxSeq = 2;
		/** 发送序号 **/
		private int seq;// 发送序号
		/** 最大关闭序号 **/
		private int closeMaxSeq = DefaultCloseMaxSeq;

		public int getSeq() {
			return seq;
		}

		public void setSeq(int seq) {
			this.seq = seq;
		}

		public int getCloseMaxSeq() {
			return closeMaxSeq;
		}

		public void setCloseMaxSeq(int closeMaxSeq) {
			this.closeMaxSeq = closeMaxSeq;
		}

	}

	@Override
	public void messageArrived(Connection conn, Object msg) {
		try {
			conn.setAttribute(LastReadTimeMills, System.currentTimeMillis());// 设置最后心跳时间
			resetHeartSeq(conn);// 设置心跳
		} catch (Exception e) {
			logger.error("resetHeart error:", e.getMessage());
		}
		if (isHeartReq(msg)) {
			sendHeartRsp(conn);
			return;
		}
		doMessageArrived(conn, msg);
	}

	@Override
	public long getReaderTimeMills() {
		return HeartIntervalMills;
	}

	@Override
	public long getWriterTimeMills() {
		return HeartIntervalMills;
	}

	@Override
	public long getAllTimeMills() {
		return getWriterTimeMills() * 4;
	}

	@Override
	public void event_AllTimeOut(Connection connection) {
		if (!isHeartEnable()) {
			return;
		}
		logger.warn("读写超时,关闭链路:" + connection);
		connection.close();
	}

	@Override
	public void event_ReadTimeOut(Connection connection) {
		if (!isHeartEnable()) {
			return;
		}
		connection.getAttribute(HeartConfig);
		HeartConfig hc = null;
		if (connection.hasAttribute(HeartConfig)) {
			hc = (HeartConfig) connection.getAttribute(HeartConfig);
		} else {
			hc = new HeartConfig();
			connection.setAttribute(HeartConfig, hc);
		}
		if (hc.getSeq() >= hc.getCloseMaxSeq()) {
			logger.warn("读超时,hc:" + hc + ",关闭链路:" + connection + ",LastReadTimeMills=" + connection.getAttribute(LastReadTimeMills));
			connection.close();
			return;
		}
		sendHeartReq(connection);
		hc.setSeq(hc.getSeq() + 1);
		logger.trace("读超时,hc:" + hc + ",发送心跳请求:" + connection);
	}

	@Override
	public void event_WriteTimeOut(Connection connection) {
		if (!isHeartEnable()) {
			return;
		}
		logger.trace("写超时,发送心跳请求:" + connection);
		sendHeartReq(connection);
	}

	/**
	 * 重置心跳发送序号
	 * 
	 * @param conn
	 */
	protected final void resetHeartSeq(Connection conn) {
		if (conn.hasAttribute(HeartConfig)) {// 重置心跳序号
			HeartConfig hc = (HeartConfig) conn.getAttribute(HeartConfig);
			hc.setSeq(0);
		}
	}

	/**
	 * 是否是心跳请求
	 * 
	 * @param msg
	 * @return
	 */
	protected abstract boolean isHeartReq(Object msg);

	/**
	 * 发送心跳请求
	 */
	protected abstract void sendHeartReq(Connection connection);

	/**
	 * 发送心跳回复
	 */
	protected abstract void sendHeartRsp(Connection connection);

	/**
	 * 处理收到的消息
	 * 
	 * @param conn
	 * @param msg
	 */
	protected abstract void doMessageArrived(Connection conn, Object msg);

}
