package com.incubator.common.net.netty.listener.adapter;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.Connection;
import com.incubator.common.net.ConnectionListener;
import com.incubator.common.net.netty.ChannelKeys;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.util.ReferenceCountUtil;
import org.slf4j.Logger;

import java.io.IOException;

/**
 * 负责chanel事件到监听器的适配
 */
abstract class AbstractListenerHandlerAdapter<T, L extends ConnectionListener<T>> extends ChannelInboundHandlerAdapter {

	protected final Logger logger = Log4jUtil.getLogger(getClass());

	protected final L listener;

	public AbstractListenerHandlerAdapter(L listener) {
		if (listener == null) {
			throw new UnsupportedOperationException("listener is null");
		}
		this.listener = listener;
	}

	@Override
	public void channelRegistered(ChannelHandlerContext ctx) throws Exception {
		super.channelRegistered(ctx);
	}

	@Override
	public final void channelUnregistered(ChannelHandlerContext ctx) throws Exception {
		super.channelUnregistered(ctx);
	}

	/** 放于心跳handler和解码器后面 **/
	@Override
	public void channelActive(ChannelHandlerContext ctx) throws Exception {
		Channel channel = ctx.channel();
		Connection connection = buildConnection(channel);
		bindConnection(channel, connection); // 绑定链接
		listener.connectionOpened(connection);
		super.channelActive(ctx);
	}

	/** 到达业务线程后需要注意msg被释放的问题 **/
	@Override
	public final void channelRead(ChannelHandlerContext ctx, Object msg) throws Exception {
		if (msg == null) {
			return;
		}
		boolean release = false;
		try {
			@SuppressWarnings("unchecked")
			T iMsg = (T) msg;
			Channel channel = ctx.channel();
			Connection connection = findConnection(channel);
			if (connection != null) {
				listener.messageArrived(connection, iMsg);
				release = true;
			} else {
				logger.error(ctx.channel() + ": 没有找到 NettyConnection 的创建...");
				// 下一个handler继续处理
				ctx.fireChannelRead(msg);
			}
		} catch (Exception e) {
			logger.error("消息出错: ", e);
			// 如果出错且还没有被释放, 下一个handler继续处理
			ctx.fireChannelRead(msg);
		} finally {
			if (release) {
				ReferenceCountUtil.release(msg);
			}
		}
	}

	@Override
	public final void channelInactive(ChannelHandlerContext ctx) throws Exception {
		Channel channel = ctx.channel();
		Connection connection = findConnection(channel);
		if (connection != null) {
			listener.connectionClosed(connection);
		} else {
			logger.error(ctx.channel() + ": 没有找到 NettyConnection 的创建...");
		}
		super.channelInactive(ctx);
	}

	@Override
	public final void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) throws Exception {
		if (cause instanceof IOException) {
			return;
		}
		logger.error(ctx.channel() + ":" + cause.toString());
		ctx.fireExceptionCaught(cause);
	}

	/**
	 * 创建一个链接实例
	 * 
	 * @return
	 */
	protected Connection buildConnection(Channel channel) {
		Connection connection = listener.getConnectionFactory().buildConnection(channel);
		if (connection == null) {
			logger.error(channel + ": 没有找到 Connection 实例...");
			throw new UnsupportedOperationException("not found Connection Instance");
		}
		return connection;
	}

	/**
	 * 绑定链接
	 * 
	 * @param channel
	 * @param conn
	 */
	protected void bindConnection(Channel channel, Connection conn) {
		channel.attr(ChannelKeys.Connection_Key).set(conn);
	}

	/**
	 * 查找链接
	 * 
	 * @param channel
	 * @return
	 */
	protected Connection findConnection(Channel channel) {
		return channel.attr(ChannelKeys.Connection_Key).get();
	}

}
