package com.incubator.common.net.netty.listener.adapter;

import com.incubator.common.net.Connection;
import com.incubator.common.net.ConnectionHeartbeatListener;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.timeout.IdleStateEvent;
import io.netty.handler.timeout.IdleStateHandler;

import java.util.concurrent.TimeUnit;

/**
 * 具有心跳机制的chanle 监听适配器 该handler必须放在编码解码器handler后面才能起作用
 * 
 * @param <M>
 */
@Sharable
public class ListenerHandlerHeartbeatAdapter<M> extends AbstractListenerHandlerAdapter<M, ConnectionHeartbeatListener<M>> {

	private ListenerIdleHandler handler;
	
	public ListenerHandlerHeartbeatAdapter(ConnectionHeartbeatListener<M> listener) {
		super(listener);
	}

	@Override
	public final void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
		if (evt instanceof IdleStateEvent) {
			Channel channel = ctx.channel();
			Connection connection = findConnection(channel);
			if (connection != null) {
				IdleStateEvent event = (IdleStateEvent) evt;
				switch (event.state()) {
				case ALL_IDLE:
					listener.event_AllTimeOut(connection);
					break;
				case READER_IDLE:
					listener.event_ReadTimeOut(connection);
					break;
				case WRITER_IDLE:
					listener.event_WriteTimeOut(connection);
					break;
				default:
					break;
				}
			} else {
				logger.error(ctx.channel() + ":not found NettyConnection Created.");
			}
		}
		super.userEventTriggered(ctx, evt);
	}
	
	@Override
	public final void handlerAdded(ChannelHandlerContext ctx) throws Exception {
		ListenerIdleHandler oldHandler = ctx.pipeline().get(ListenerIdleHandler.class);
		if (oldHandler != null) {
			logger.error("old Handler:" + oldHandler);
		}
		handler = new ListenerIdleHandler(listener.getReaderTimeMills(), listener.getWriterTimeMills(), listener.getAllTimeMills(), TimeUnit.MILLISECONDS);
		ctx.pipeline().addBefore(ctx.name(), getIdleHandlerName(ctx), handler); // 当前ctx名字就是当前handler加入pipe的名字
		super.handlerAdded(ctx);
	}

	/**
	 * 和监听器相关的读写超时监测handler 不使用IdleStateHandler以防影响其它扩展
	 * 
	 */
	static class ListenerIdleHandler extends IdleStateHandler {
		
		public ListenerIdleHandler(int readerIdleTimeSeconds, int writerIdleTimeSeconds, int allIdleTimeSeconds) {
			super(readerIdleTimeSeconds, writerIdleTimeSeconds, allIdleTimeSeconds);
		}

		public ListenerIdleHandler(long readerIdleTime, long writerIdleTime, long allIdleTime, TimeUnit unit) {
			super(readerIdleTime, writerIdleTime, allIdleTime, unit);
		}
		
	}

	protected String getIdleHandlerName(ChannelHandlerContext ctx) {
		return "ListenerIdleHandle(" + ctx.channel().hashCode() + ")";
	}

}
