package com.incubator.common.net.netty.server;

import java.net.InetSocketAddress;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.netty.ServerOptionConfiger;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.group.ChannelGroup;
import io.netty.channel.group.DefaultChannelGroup;
import io.netty.util.concurrent.ImmediateEventExecutor;

/**
 * netty服
 * 
 * @author johnc
 *
 */
public class NettyServer extends NettyServerFactory {

	private final Logger logger = Log4jUtil.getLogger(getClass());

	protected final ChannelGroup channelGroup = new DefaultChannelGroup(getName() + "ChannelGroup", ImmediateEventExecutor.INSTANCE);

	protected final ServerBootstrap booter = new ServerBootstrap();

	protected final NettyServerConfig config;

	protected final ChannelHandler handler;

	public NettyServer(String host, int port, ChannelHandler handler) {
		this(new InetSocketAddress(host, port), handler);
	}

	public NettyServer(InetSocketAddress local, ChannelHandler handler) {
		this(new NettyServerConfig(), local, handler);
	}

	public NettyServer(NettyServerConfig config, InetSocketAddress local, ChannelHandler handler) {
		super(local);
		this.config = config;
		this.handler = handler;
		initBooter();
	}

	/** 初始化Booter **/
	private void initBooter() {
		this.booter.group(this.config.getBoss(), this.config.getIoworkers());
		this.booter.channel(this.config.getChannelClass());
	}

	@Override
	public final void broadCast(Object message) {
		this.channelGroup.writeAndFlush(message);
		logger.debug("broadCast message total:" + this.channelGroup.size() + ",type:" + message);
	}

	@Override
	public int getConnectionCount() {
		return this.channelGroup.size();
	}

	@Override
	protected final ChannelFuture doBind(InetSocketAddress local) {
		this.booter.localAddress(local);
		initServerOptions(optionConfig());
		ChannelHandler fixedHandler = fixHandlerBeforeDoBooterBind(this.handler);// 修正handler
		return doBooterBind(local, fixedHandler);// 启动端口绑定
	}

	@Override
	public ServerBootstrap getBooter() {
		return this.booter;
	}

	@Override
	public EventLoopGroup getIoWorkers() {
		return this.config.boss;
	}

	@Override
	public EventLoopGroup getBossWorkers() {
		return this.config.ioWorkers;
	}

	public ChannelGroup getChannelGroup() {
		return this.channelGroup;
	}

	protected void initServerOptions(ServerOptionConfiger configer) {
		configer.option(ChannelOption.SO_BACKLOG, 1024);// 设置连接等待最大队列
		//configer.option(ChannelOption.TCP_NODELAY, true);
		//configer.option(ChannelOption.SO_KEEPALIVE, true);// 设置保持连接
	}

	public ServerOptionConfiger optionConfig() {
		return new ServerOptionConfiger() {
			@Override
			public <T> ServerOptionConfiger option(ChannelOption<T> option, T value) {
				booter.option(option, value);
				return this;
			}
			@Override
			public <T> ServerOptionConfiger childOption(ChannelOption<T> option, T value) {
				booter.childOption(option, value);
				return this;
			}
		};
	}

	/**
	 * 执行启动绑定之前修正handler,子类可进行二次修改
	 * 
	 * @param handler
	 * @return
	 */
	protected ChannelHandler fixHandlerBeforeDoBooterBind(ChannelHandler handler) {
		return handler;
	}

	/**
	 * 绑定
	 * 
	 * @param local
	 * @param fixedHandler
	 * @return
	 */
	protected ChannelFuture doBooterBind(InetSocketAddress local, final ChannelHandler fixedHandler) {
		ChannelFuture cf = null;
		synchronized (this.booter) {
			final CountDownLatch latch = new CountDownLatch(1);
			this.booter.childHandler(new ChannelInitializer<Channel>() {
				@Override
				protected void initChannel(Channel ch) throws Exception {
					channelGroup.add(ch);
					ch.pipeline().addLast(fixedHandler);
				}
			});
			cf = this.booter.bind(local);
			cf.addListener(new ChannelFutureListener() {
				@Override
				public void operationComplete(ChannelFuture future) throws Exception {
					latch.countDown();
				}
			});
			try {
				latch.await(3, TimeUnit.SECONDS);
			} catch (Exception e) {
				logger.error("=====绑定等待出错: {}", e.getMessage());
			}
		}
		return cf;
	}

}
