package com.incubator.core.db.handler;

import com.incubator.core.db.DBBase;

import java.sql.*;
import java.util.ArrayList;
import java.util.List;

/**
 * 数据库持久化操纵基类, 使用之前必须DBFactory.init();
 * @author agui
 */
public abstract class DBHandler extends DBBase {
	protected DBHandler() {
	}
			
	protected int update(String sql) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		try {
			conn = getMaster();
			pstmt = conn.prepareStatement(sql);
			return pstmt.executeUpdate();
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(null, pstmt, conn);
		}
		return -1;
	}
	
	/**
	 * 变更
	 */
	protected int update(String sql, Object... args) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		try {
			conn = getMaster();
			pstmt = conn.prepareStatement(sql);
			setParams(pstmt, args);
			return pstmt.executeUpdate();
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(null, pstmt, conn);
		}
		return -1;
	}
	
	/**
	 * 变更
	 */
	protected int update(String sql, ParamMapper param) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		try {
			conn = getMaster();
			pstmt = conn.prepareStatement(sql);
			param.setParams(pstmt, 0);
			return pstmt.executeUpdate();
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(null, pstmt, conn);
		}
		return -1;
	}

	
	/**
	 * 保存或者更新
	 * return 1:保存，0：更新
	 */
	protected int saveOrUpdate(ISaveOrUpdate iSaveOrUpdate) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		try {
			conn = getMaster();
			pstmt = conn.prepareStatement(iSaveOrUpdate.getUpdateSQL());
			setParams(pstmt, iSaveOrUpdate.getUpdateParams());
			if (pstmt.executeUpdate() == 0) {
				close(pstmt);
				pstmt = conn.prepareStatement(iSaveOrUpdate.getSaveSQL());
				setParams(pstmt, iSaveOrUpdate.getSaveParams());
				return pstmt.executeUpdate();
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(null, pstmt, conn);
		}
		return 0;
	}
	
	/**
	 * 批量变更
	 */
	protected void batchUpdate(String sql, List<Object[]> batch) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		try {
			conn = getMaster();
			pstmt = conn.prepareStatement(sql);
			for (Object[] par : batch) {
				setParams(pstmt, par);
				pstmt.addBatch();
			}
			pstmt.executeBatch();
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(null, pstmt, conn);
		}
	}
	
	/**
	 * 批量变更
	 */
	protected void batchUpdate(String sql, ParamMapper mapper, int size) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		try {
			conn = getMaster();
			pstmt = conn.prepareStatement(sql);
			for (int row = 0; row < size; row++) {
				mapper.setParams(pstmt, row);
				pstmt.addBatch();
				if (row % 500 == 0) {
					pstmt.executeBatch();
					pstmt.clearBatch();
				}
			}
			if(size % 500 != 0) {
				pstmt.executeBatch();
				pstmt.clearBatch();
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(null, pstmt, conn);
		}
	}
	
	/**
	 * 查询集合
	 */
	protected <T> List<T> query(String sql, RowMapper<T> rowMapper, Object... args) {
		List<T> values = new ArrayList<T>();
		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getMaster();
			pstmt = conn.prepareStatement(sql);
			setParams(pstmt, args);
			rs = pstmt.executeQuery();
			while (rs.next()) {
				values.add(rowMapper.mapRow(rs));
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, pstmt, conn);
		}
		return values;
	}
	
	/**
	 * 查询集合，自定义PreparedStatement的参数<br>
	 * 需要重载setParams方法
	 */
	protected <T> List<T> query(String sql, RowMapper<T> rowMapper) {
		List<T> values = new ArrayList<T>();
		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getSlave();
			pstmt = conn.prepareStatement(sql);
			rowMapper.setParams(pstmt);
			rs = pstmt.executeQuery();
			while (rs.next()) {
				values.add(rowMapper.mapRow(rs));
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, pstmt, conn);
		}
		return values;
	}
	
	/**
	 * 查询集合
	 */
	protected void query(String sql, RowReader rowReader, Object... args) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getSlave();
			pstmt = conn.prepareStatement(sql);
			setParams(pstmt, args);
			rs = pstmt.executeQuery();
			while (rs.next()) {
				rowReader.mapRow(rs);
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, pstmt, conn);
		}
	}
	
	/**
	 * 查询集合，自定义PreparedStatement的参数<br>
	 * 需要重载setParams方法
	 */
	protected void query(String sql, RowReader rowReader) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getSlave();
			pstmt = conn.prepareStatement(sql);
			rowReader.setParams(pstmt);
			rs = pstmt.executeQuery();
			while (rs.next()) {
				rowReader.mapRow(rs);
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, pstmt, conn);
		}
	}

	protected int queryForInt(String sql, RowMapper<Integer> rowMapper) {
		Integer value = queryForObject(sql, rowMapper);
		return value == null ? 0 :  value.intValue();
	}
	
	protected int queryForInt(String sql, RowMapper<Integer> rowMapper, Object... args) {
		Integer value = queryForObject(sql, rowMapper, args);
		return value == null ? 0 :  value.intValue();
	}

	protected long queryForLong(String sql, RowMapper<Long> rowMapper) {
		Long value = queryForObject(sql, rowMapper);
		return value == null ? 0 :  value.longValue();
	}

	protected long queryForLong(String sql, RowMapper<Long> rowMapper, Object... args) {
		Long value = queryForObject(sql, rowMapper, args);
		return value == null ? 0 :  value.longValue();
	}
	
	/**
	 * 查询单体
	 */
	protected <T> T queryForObject(String sql, RowMapper<T> rowMapper, Object... args) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getSlave();
			pstmt = conn.prepareStatement(sql);
			setParams(pstmt, args);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				return rowMapper.mapRow(rs);
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, pstmt, conn);
		}
		return null;
	}
	
	/**
	 * 查询单体，自定义PreparedStatement的参数<br>
	 * 需要重载setParams方法
	 */
	protected <T> T queryForObject(String sql, RowMapper<T> rowMapper) {
		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getSlave();
			pstmt = conn.prepareStatement(sql);
			rowMapper.setParams(pstmt);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				return rowMapper.mapRow(rs);
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, pstmt, conn);
		}
		return null;
	}
	
	/** 查询是否存在 */
	protected boolean queryExists(String sql, Object... args) {
		
		Connection conn = null;
		PreparedStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getSlave();
			pstmt = conn.prepareStatement(sql);
			setParams(pstmt, args);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				return true;
			}
		} catch (SQLException e) {
			exceptionSQL(e);
			return false;
		} finally {
			free(rs, pstmt, conn);
		}
		
		return false;
	}
	
	/** 执行存储过程 , 无返回*/
	protected boolean excuteProcess(String sql, Object... args) {		
		Connection conn = null;
		CallableStatement cstmt = null;
		ResultSet rs = null;
		try {
			conn = getMaster();
			cstmt = conn.prepareCall(sql);
			setParams(cstmt, args);
			rs = cstmt.executeQuery();
			return true;
		} catch (SQLException e) {
			exceptionSQL(sql, e, args);
		} finally {
			free(rs, cstmt, conn);
		}
		
		return false;
	}
	
	/** 执行存储过程 , 无返回*/
	protected void excuteProcess(String sql, RowReader rowReader, Object... args) {
		Connection conn = null;
		CallableStatement cstmt = null;
		ResultSet rs = null;
		try {
			conn = getMaster();
			cstmt = conn.prepareCall(sql);
			setParams(cstmt, args);
			rs = cstmt.executeQuery();
			while (rs.next()) {
				rowReader.mapRow(rs);
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, cstmt, conn);
		}
	}	
	
	/** 执行存储过程，返回单体 */
	protected int excuteProcessForInt(String sql, Object... args) {
		Connection conn = null;
		CallableStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getSlave();
			pstmt = conn.prepareCall(sql);
			setParams(pstmt, args);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				return rs.getInt(1);
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, pstmt, conn);
		}
		return -1;
	}

	/** 执行存储过程，返回单体 */
	protected long excuteProcessForLong(String sql, Object... args) {
		Connection conn = null;
		CallableStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getSlave();
			pstmt = conn.prepareCall(sql);
			setParams(pstmt, args);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				return rs.getLong(1);
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, pstmt, conn);
		}
		return -1;
	}

	/** 执行存储过程，返回单体 */
	protected String excuteProcessForString(String sql, Object... args) {
		Connection conn = null;
		CallableStatement pstmt = null;
		ResultSet rs = null;
		try {
			conn = getSlave();
			pstmt = conn.prepareCall(sql);
			setParams(pstmt, args);
			rs = pstmt.executeQuery();
			if (rs.next()) {
				return rs.getString(1);
			}
		} catch (SQLException e) {
			exceptionSQL(e);
		} finally {
			free(rs, pstmt, conn);
		}
		return "";
	}
}
