package com.incubator.core.db.pool;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.xml.XW3CParser;
import com.incubator.core.util.GConst;
import org.slf4j.Logger;

import javax.naming.NameAlreadyBoundException;
import javax.naming.NameNotFoundException;
import javax.sql.DataSource;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 连接池类厂，该类常用来保存多个数据源名称和数据库连接池对应的哈希
 * 
 * @author agui
 */
public final class DBFactory {

	private static final Logger logger = Log4jUtil.getLogger(DBFactory.class);

	/**
	 * 是否已初始化
	 */
	private static boolean isInit;

	/**
	 * 该哈希表用来保存数据源名和连接池对象的关系表
	 */
	static Map<String, DBSource> dsPools = null;

	/**
	 * 从连接池工厂中获取指定名称对应的连接池对象
	 * 
	 * @param dataSource 连接池对象对应的名称
	 * @return DataSource 返回名称对应的连接池对象
	 * @throws NameNotFoundException 无法找到指定的连接池
	 */
	public static DBSource lookup(String dataSource) {
		DBSource ds = null;
		if (dsPools != null) {
			ds = dsPools.get(dataSource);
		}
		return ds;
	}

	/**
	 * 将指定的名字和数据库连接配置绑定在一起并初始化数据库连接池
	 * 
	 * @param name  对应连接池的名称
	 * @param param 连接池的配置参数，具体请见类ConnectionParam
	 * @return DataSource 如果绑定成功后返回连接池对象
	 * @throws NameAlreadyBoundException 如果name已经绑定则抛出该异常
	 * @throws ClassNotFoundException    无法找到连接池的配置中的驱动程序类
	 * @throws IllegalAccessException    连接池配置中的驱动程序类有误
	 * @throws InstantiationException    无法实例化驱动程序类
	 * @throws SQLException              无法正常连接指定的数据库
	 */
	public static DBSource start(String name, DBAttribute param) throws NameAlreadyBoundException, SQLException {
		if (lookup(name) != null) {
			throw new NameAlreadyBoundException(name);
		}
		DBSource source = new DBSource(param);
		if (dsPools == null) {
			dsPools = new ConcurrentHashMap<>();
		}
		dsPools.put(name, source);
		return source;
	}

	/**
	 * 重新绑定数据库连接池
	 * 
	 * @param name  对应连接池的名称
	 * @param param 连接池的配置参数，具体请见类ConnectionParam
	 * @return DataSource 如果绑定成功后返回连接池对象
	 * @throws NameAlreadyBoundException 一定名字name已经绑定则抛出该异常
	 * @throws ClassNotFoundException    无法找到连接池的配置中的驱动程序类
	 * @throws IllegalAccessException    连接池配置中的驱动程序类有误
	 * @throws InstantiationException    无法实例化驱动程序类
	 * @throws SQLException              无法正常连接指定的数据库
	 */
	public static DBSource restart(String name, DBAttribute param) throws NameAlreadyBoundException,
			ClassNotFoundException, IllegalAccessException, InstantiationException, SQLException {
		try {
			shutdown(name);
		} catch (Exception e) {
		}
		return start(name, param);
	}

	/**
	 * 删除一个数据库连接池对象
	 * 
	 * @param name
	 * @throws NameNotFoundException
	 */
	public static void shutdown(String name) throws NameNotFoundException {
		if (dsPools == null) {
			return;
		}
		DBSource dataSource = lookup(name);

		if (dataSource != null) {
			DBSource datasource = dataSource;
			datasource.shutdown();
			datasource = null;
		}

		dsPools.remove(name);
	}

	/**
	 * 从配置文件加载数据源
	 */
	public static void init() {
		init(GConst.FILE_CONF_DS);
	}

	/**
	 * 从配置文件加载数据源
	 */
	public synchronized static String init(String xmlFile) {
		if (!isInit) {
			// 读取数据库配置文件
			dsPools = new ConcurrentHashMap<String, DBSource>(2, 0.75F);
			List<DBAttribute> drivers = null;
			try {
				drivers = XW3CParser.parse(xmlFile, DBAttribute.class);
			} catch (Exception e) {
				logger.error("读取连接池配置文件错误 -> {}", xmlFile);
				e.printStackTrace();
				System.exit(-1);
			}

			if (drivers.size() == 0) {
				logger.warn("连接池配置文件中未配置 -> {}", xmlFile);
				return "连接池配置文件中未配置 -> ," + xmlFile;
			}

			StringBuilder dsInfo = new StringBuilder();
			DBAttribute cp = null;

			for (int i = 0; i < drivers.size(); i++) {
				try {
					cp = drivers.get(i);
					start(cp.getDsName(), cp);
					dsInfo.append(cp.getDsName());

					if (i < drivers.size() - 1) {
						dsInfo.append(" & ");
					}
				} catch (Exception e) {
					e.printStackTrace();
					logger.error("绑定连接池 [{}] 错误！", drivers.get(i).getDsName());
					System.exit(-1);
				}
			}
			isInit = true;
			logger.info(String.format("初始化成功连接池数量: %d, 数据库连接池【 %s 】", drivers.size(), dsInfo.toString()));
			return String.format("初始化成功连接池数量:%d, 数据库连接池【 %s 】", drivers.size(), dsInfo.toString());
		}
		return "连接池配置文件已初始化 -> ," + xmlFile;
	}

}