package com.incubator.core.net.ws;

import java.util.ArrayList;
import java.util.Map;

public class WSRequest extends MsgBase {

    public WSRequest(){}

    public WSRequest(byte[] bytes) throws Exception {
        this.fromBytes(bytes);
    }

    /**
     * 根据 key 获取 data 字段的值，返回默认值如果不存在或类型不匹配
     *
     * @param key   要获取的 key
     * @param clazz 期望的返回值类型的 Class 对象
     * @param defaultValue 默认值
     * @param <T>   泛型，用于指定返回值类型
     * @return 返回 key 对应的值，如果不存在或类型不匹配则返回 defaultValue
     */
    public <T> T getDataValue(String key, Class<T> clazz, T defaultValue) {
        try {
            return this.getDataValue(key, clazz);
        } catch (IllegalArgumentException e) {
            return defaultValue;
        }
    }

    /**
     * 根据 key 获取 data 字段的值，并确保类型匹配
     *
     * @param key   要获取的 key
     * @param clazz 期望的返回值类型的 Class 对象
     * @param <T>   泛型，用于指定返回值类型
     * @return 返回 key 对应的值，如果不存在或类型不匹配则返回 null
     * @throws IllegalArgumentException 如果类型不匹配
     */
    private  <T> T getDataValue(String key, Class<T> clazz) {
        if (key == null || clazz == null) {
            throw new IllegalArgumentException("Key and clazz cannot be null");
        }

        Object value = this.data.get(key);

        // 如果值为 null，直接返回 null
        if (value == null) {
            return null;
        }

        // 处理基本数据类型 (int, long, double, float, boolean, char)
        if (isBasicType(clazz)) {
            return handleBasicType(value, clazz);
        }

        // 处理数组类型（包括 int[], Integer[], String[] 等）
        if (clazz.isArray()) {
            return handleArrayType(value, clazz);
        }

        // 处理集合类型 (ArrayList)
        if (clazz == ArrayList.class && value instanceof ArrayList) {
            return clazz.cast(value);
        }

        // 处理 Map 类型
        if (clazz == Map.class && value instanceof Map) {
            return clazz.cast(value);
        }

        // 如果类型匹配，直接返回
        if (clazz.isInstance(value)) {
            return clazz.cast(value);
        }

        // 类型不匹配，抛出异常
        throw new IllegalArgumentException(
                String.format("Value for key '%s' is not of type %s, but of type %s",
                        key, clazz.getName(), value.getClass().getName()));
    }

    /**
     * 判断是否为基本类型
     */
    private boolean isBasicType(Class<?> clazz) {
        return clazz == Integer.class || clazz == int.class ||
                clazz == Long.class || clazz == long.class ||
                clazz == Double.class || clazz == double.class ||
                clazz == Float.class || clazz == float.class ||
                clazz == Boolean.class || clazz == boolean.class ||
                clazz == Character.class || clazz == char.class ||
                clazz == String.class;
    }

    /**
     * 处理基本数据类型 (Integer, Long, Double, Float, Boolean, Character)
     */
    private <T> T handleBasicType(Object value, Class<T> clazz) {
        // 处理数值类型 (Integer, Long, Double, Float)
        if (value instanceof Number) {
            Number numberValue = (Number) value;

            if (clazz == Integer.class || clazz == int.class) {
                return clazz.cast(numberValue.intValue());
            } else if (clazz == Long.class || clazz == long.class) {
                return clazz.cast(numberValue.longValue());
            } else if (clazz == Double.class || clazz == double.class) {
                return clazz.cast(numberValue.doubleValue());
            } else if (clazz == Float.class || clazz == float.class) {
                return clazz.cast(numberValue.floatValue());
            }
        }

        // 处理 Boolean 类型
        if (clazz == Boolean.class || clazz == boolean.class) {
            if (value instanceof Boolean) {
                return clazz.cast(value);
            } else if (value instanceof Number) {
                return clazz.cast(((Number) value).intValue() != 0);
            } else if (value instanceof String) {
                return clazz.cast(Boolean.parseBoolean((String) value));
            }
        }

        // 处理 Character 类型
        if (clazz == Character.class || clazz == char.class) {
            if (value instanceof String && ((String) value).length() == 1) {
                return clazz.cast(((String) value).charAt(0));
            }
        }

        // 处理 String 类型
        if (clazz == String.class) {
            // 如果值已经是 String，直接返回
            if (value instanceof String) {
                return clazz.cast(value);
            }

            // 否则尝试将其他类型转换为 String
            return clazz.cast(value.toString());
        }

        return null;
    }

    /**
     * 处理数组类型 (int[], Integer[], String[], etc.)
     */
    private <T> T handleArrayType(Object value, Class<T> clazz) {
        if (value instanceof ArrayList) {
            ArrayList<?> list = (ArrayList<?>) value;

            // 处理 int[] 和 Integer[]
            if (clazz == int[].class) {
                return clazz.cast(list.stream().mapToInt(item -> ((Number) item).intValue()).toArray());
            } else if (clazz == Integer[].class) {
                return clazz.cast(list.stream().map(item -> ((Number) item).intValue()).toArray(Integer[]::new));
            } else if (clazz == String[].class) {
                return clazz.cast(list.toArray(new String[0]));
            } else if (clazz == Double[].class) {
                return clazz.cast(list.stream().map(item -> ((Number) item).doubleValue()).toArray(Double[]::new));
            } else if (clazz == long[].class) {
                return clazz.cast(list.stream().mapToLong(item -> ((Number) item).longValue()).toArray());
            } else if (clazz == Long[].class) {
                return clazz.cast(list.stream().map(item -> ((Number) item).longValue()).toArray(Long[]::new));
            } else if (clazz == Float[].class) {
                return clazz.cast(list.stream().map(item -> ((Number) item).floatValue()).toArray(Float[]::new));
            }
        }

        // 处理原生数组
        if (value.getClass().isArray()) {
            return clazz.cast(value);
        }

        return null;
    }
}
