package com.incubator.game;

import com.incubator.common.MessageHandler;
import com.incubator.common.env.Environment;
import com.incubator.common.game.AbstractService;
import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.Connection;
import com.incubator.common.net.NetClient;
import com.incubator.common.net.NetServer;
import com.incubator.common.net.netty.client.NettyClient;
import com.incubator.common.net.netty.handler.websocket.binary.WebSocketServerBinaryAdapterHandler;
import com.incubator.common.net.netty.server.NettyServer;
import com.incubator.common.thread.NamedThreadFactory;
import com.incubator.common.util.PropertiesUtil;
import com.incubator.core.net.http.HttpRequestHandler;
import com.incubator.core.quartz.QuartzServer;
import com.incubator.core.quartz.QuartzTask;
import com.incubator.game.listener.CenterClientListener;
import com.incubator.game.listener.HttpInnerListener;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.handler.InitalizerHttpServerHandler;
import com.incubator.core.net.handler.InitalizerProtocolHandler;
import com.incubator.game.data.entity.GlobalInfoPO;
import com.incubator.game.data.jdbc.LSDBHandler;
import com.incubator.game.listener.PublicListener;
import com.incubator.game.player.GPlayer;
import com.incubator.game.player.PlayerPool;
import com.incubator.game.room.RoomManager;
import com.incubator.game.timer.FlushDB;
import com.incubator.game.timer.RefreshNewDay;
import com.incubator.game.util.GameDataUtil;
import org.apache.commons.lang3.StringUtils;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.reflections.Reflections;
import org.slf4j.Logger;

import java.io.File;
import java.io.FileInputStream;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

public class GGame extends AbstractService {

    private static final Logger logger;

    public static ScheduledThreadPoolExecutor scheduleExec = new ScheduledThreadPoolExecutor(2, new NamedThreadFactory("ServerExecutor", true));
    /** 运行环境 **/
    public static final Environment env = Environment.getInstance();
    private static final LSDBHandler lsDBHandler = new LSDBHandler();
    /** 服务器设置目录 **/
    public static PropertiesUtil SERVER_SETTINGS;
    public static PropertiesUtil DB_SETTINGS;
    public static String CONFIG_FILE = "config.properties";
    public static String DB_FILE = "db.properties";

    /** 服务器id **/
    public static int SERVER_ID = 1001;
    /** 服务器绑定IP **/
    public static String BIND_HOST;
    /** 服务器绑定端口 **/
    public static int BIND_PORT;
    /** 绑定的http ip **/
    public static String BIND_HTTP_HOST;
    /** 绑定的http 端口 **/
    public static int BIND_HTTP_PORT;
    /** 中控服绑定ip **/
    public static String CENTER_HOST;
    /** 中控服绑定端口 **/
    public static int CENTER_PORT;
    /** 策划JSON数据目录 **/
    public static String DIR_GAME_DATA = "";

    /** 玩家心跳开关 **/
    public static boolean heartEnable = false;
    /** 最大空闲玩家数量 **/
    public static int maxIdlePlayers = 100;

    /** 游戏世界的时间
     *  此两个时间最小精度为秒,不能用于超过精度一秒的时间计算
     *  (APP_TIME每隔一秒执行一下,所以最小精度一秒),方法为JobFactory.addScheduleJob
     * */
    public static volatile long APP_TIME = System.currentTimeMillis();

    public static volatile int APP_TIME_SEC = (int) (APP_TIME / 1000);


    /** 是否注册中控 **/
    public static boolean isRegCenter;

    /** 存储连接 **/
    public static Set<Connection> publicClients = new HashSet<>();

    /** ws消息事件 **/
    public static Map<Integer, NetHandler> handlers = new HashMap<>();

    /** http消息事件 **/
    public static Map<String, HttpRequestHandler> httpHandlers = new HashMap<>();

    /** 玩家对象池 **/
    public static PlayerPool playerPool = new PlayerPool();

    /** 本服所有在线玩家 */
    public static Map<String, GPlayer> onlinePlayers = new ConcurrentHashMap<>();

    public static GlobalInfoPO globalInfoPO = new GlobalInfoPO();

    private static volatile GGame instance;

    static {
        // 日志加载
        Log4jUtil.initLogConfig(env.getLogConfigFile());
        logger = Log4jUtil.getLogger(GGame.class);
    }

    public static GGame getInstance() {
        if (instance == null) {
            synchronized (GGame.class) {
                if (instance == null) {
                    instance = new GGame();
                }
            }
        }
        return instance;
    }

    public static void addPlayer(GPlayer player) {
        onlinePlayers.put(player.getId(), player);
    }

    public static LSDBHandler getDBHandler() {
        return lsDBHandler;
    }

    @Override
    protected void doStart() throws Throwable {
        // 1.初始化Properties配置
        this.initProperties();
        // 2.初始化数据库连接池
//        DBFactory.init();
        // 3.初始化游戏数据
        this.initGameData();
        // 4.初始化定时任务
        this.initQuartzTask();
        // 5.初始化消息事件
        this.initHandler();
        // 6.初始化网络服务
        this.initNetService();
    }

    @Override
    protected void doClose() throws Throwable {
        logger.info("服务器维护中, 已关闭服务器!");
    }

    /**
     * 1. 初始化 Properties 配置
     *
     * @throws Exception 如果配置加载失败
     */
    public void initProperties() throws Exception {
        logger.info("开始加载 Properties 配置...");

        try {
            // 加载服务器配置
            SERVER_SETTINGS = loadProperties(CONFIG_FILE);
            DB_SETTINGS = loadProperties(DB_FILE);

            // 读取服务器配置
            SERVER_ID = getIntProperty(SERVER_SETTINGS, "SERVER_ID", 1001);
            BIND_HOST = getStringProperty(SERVER_SETTINGS, "BIND_HOST", "0.0.0.0");
            BIND_PORT = getIntProperty(SERVER_SETTINGS, "BIND_PORT", 9000);
            BIND_HTTP_HOST = getStringProperty(SERVER_SETTINGS, "BIND_HTTP_HOST", "0.0.0.0");
            BIND_HTTP_PORT = getIntProperty(SERVER_SETTINGS, "BIND_HTTP_PORT", 9001);
            CENTER_HOST = getStringProperty(SERVER_SETTINGS, "CENTER_HOST", "0.0.0.0");
            CENTER_PORT = getIntProperty(SERVER_SETTINGS, "CENTER_PORT", 4000);
            DIR_GAME_DATA = getStringProperty(SERVER_SETTINGS, "DIR_GAME_DATA", "");

            logger.info("Properties 配置加载完成");
        } catch (Exception e) {
            logger.error("加载 Properties 配置失败: {}", e.getMessage(), e);
            throw new Exception("Properties 配置加载失败", e);
        }
    }

    /**
     * 加载 Properties 文件
     *
     * @param fileName 文件名
     * @return Properties 对象
     * @throws Exception 如果加载失败
     */
    private PropertiesUtil loadProperties(String fileName) throws Exception {
        String filePath = env.getConfDir() + "/" + fileName;
        logger.info("加载配置文件: {}", filePath);
        return new PropertiesUtil(filePath);
    }

    /**
     * 获取 String 类型属性值
     *
     * @param properties Properties 对象
     * @param key 属性键
     * @param defaultValue 默认值
     * @return 属性值（如果属性不存在，则返回默认值）
     */
    private String getStringProperty(PropertiesUtil properties, String key, String defaultValue) {
        return properties.getProperty(key, defaultValue);
    }

    /**
     * 获取 int 类型属性值
     *
     * @param properties Properties 对象
     * @param key 属性键
     * @param defaultValue 默认值
     * @return 属性值（如果属性不存在或格式错误，则返回默认值）
     */
    private int getIntProperty(PropertiesUtil properties, String key, int defaultValue) {
        try {
            return Integer.parseInt(properties.getProperty(key, String.valueOf(defaultValue)));
        } catch (NumberFormatException e) {
            logger.warn("属性 {} 格式错误，使用默认值 {}", key, defaultValue);
            return defaultValue;
        }
    }

    /**
     * 3.初始化游戏数据
     */
    public void initGameData() {
        //加载策划配置表
        GameDataUtil.initData();
        //加载数据库数据
//        this.initAllBlobGameData();
        //初始化 房间服务
        RoomManager.getInstance().init(5);
    }

//    private ProtoData.GlobalAllBlobData.Builder initAllBlobGameData() {
//        ProtoData.GlobalAllBlobData.Builder allBlobData = ProtoData.GlobalAllBlobData.newBuilder();
//        try {
//            globalInfoPO = getDBHandler().getGlobalInfoByServer(SERVER_ID);
//            if (globalInfoPO == null) {
//                globalInfoPO = new GlobalInfoPO();
//                globalInfoPO.server_id = SERVER_ID;
//            }
//            if (globalInfoPO.allBlobData == null) {
//                globalInfoPO.allBlobData = ProtoData.GlobalAllBlobData.newBuilder().build().toByteArray();
//                getDBHandler().insertGlobal(globalInfoPO);
//            }
//            allBlobData = ProtoData.GlobalAllBlobData.parseFrom(globalInfoPO.allBlobData).toBuilder();
//        } catch (InvalidProtocolBufferException e) {
//            logger.error("初始化服务器数据出错...{}", e.getMessage());
//        }
//        return allBlobData;
//    }

    /**
     * 4. 初始化定时器任务
     */
    private void initQuartzTask() {
        logger.info("开始初始化定时任务...");
        try {
            QuartzServer quartzServer = new QuartzServer();

            // 定期刷新数据库任务
            addScheduledTask(quartzServer, new FlushDB(), "刷新数据库任务");

            // 每日刷新任务
            addScheduledTask(quartzServer, new RefreshNewDay(), "每日刷新任务");

            // 如果需要添加其他任务，可以在此处扩展
            // addScheduledTask(quartzServer, new OtherTask(), "其他任务");

            logger.info("定时任务初始化完成");
        } catch (Exception e) {
            logger.error("初始化定时任务时发生错误: {}", e.getMessage(), e);
        }
    }

    /**
     * 添加单个定时任务
     * @param quartzServer Quartz 任务管理器
     * @param job 任务对象
     * @param taskDescription 任务描述（用于日志输出）
     */
    private void addScheduledTask(QuartzServer quartzServer, QuartzTask job, String taskDescription) {
        try {
            quartzServer.addJob(job);
            logger.info("{} 已成功注册", taskDescription);
        } catch (Exception e) {
            logger.error("{} 注册失败: {}", taskDescription, e.getMessage(), e);
        }
    }

    /**
     * 5. 初始化消息事件
     */
    private void initHandler() {
        logger.info("初始化消息事件（基于注解）...");

        try {
            // 使用 Reflections 库扫描指定包
            Reflections reflections = new Reflections("com.incubator.game.handler");

            // 扫描 WebSocket 事件
            Set<Class<?>> annotatedClasses = reflections.getTypesAnnotatedWith(MessageHandler.class);
            for (Class<?> clazz : annotatedClasses) {
                MessageHandler annotation = clazz.getAnnotation(MessageHandler.class);
                if (annotation == null) continue;

                // 注册 WebSocket 事件
                if (annotation.id() > 0 && NetHandler.class.isAssignableFrom(clazz)) {
                    NetHandler handler = (NetHandler) clazz.getDeclaredConstructor().newInstance();
                    registerWebSocketHandler(annotation.id(), handler);
                }

                // 注册 HTTP 事件
                if (!annotation.httpPath().isEmpty() && HttpRequestHandler.class.isAssignableFrom(clazz)) {
                    HttpRequestHandler handler = (HttpRequestHandler) clazz.getDeclaredConstructor().newInstance();
                    registerHttpHandler(annotation.httpPath(), handler);
                }
            }

            logger.info("消息事件初始化完成");
        } catch (Exception e) {
            logger.error("初始化消息事件时发生异常: {}", e.getMessage(), e);
        }
    }

    /**
     * 注册 WebSocket 消息事件
     */
    private void registerWebSocketHandler(int id, NetHandler handler) {
        if (handlers.containsKey(id)) {
            logger.warn("WebSocket 消息事件已存在: id={}, handler={}", id, handler.getClass().getSimpleName());
            return;
        }
        handlers.put(id, handler);
        logger.info("已注册 WebSocket 消息事件: id={}, handlerClass={}", id, handler.getClass().getSimpleName());
    }

    /**
     * 注册 HTTP 消息事件
     */
    private void registerHttpHandler(String path, HttpRequestHandler handler) {
        if (httpHandlers.containsKey(path)) {
            logger.warn("HTTP 消息事件已存在: path={}, handler={}", path, handler.getClass().getSimpleName());
            return;
        }
        httpHandlers.put(path, handler);
        logger.info("已注册 HTTP 消息事件: path={}, handlerClass={}", path, handler.getClass().getSimpleName());
    }

    /**
     * 6. 初始化网络服务
     */
    private void initNetService() {
        logger.info("开始初始化网络服务...");

        try {
            // 初始化 WebSocket 服务
            initializeWebSocketServer();
            // 初始化 HTTP 服务
            initializeHttpServer();
            // 初始化中控连接服务（可选）
//            initializeCenterService();
            logger.info("网络服务初始化完成");
        } catch (Exception e) {
            logger.error("初始化网络服务时发生错误: {}", e.getMessage(), e);
        }
    }

    /**
     * 初始化 WebSocket 服务
     */
    private void initializeWebSocketServer() throws Exception {
        logger.info("初始化 WebSocket 服务...");
        NetServer publicServer = new NettyServer(
                BIND_HOST,
                BIND_PORT,
                new WebSocketServerBinaryAdapterHandler(
                        "/ws",
                        new InitalizerProtocolHandler(new PublicListener())
                )
        );
        publicServer.start();
        logger.info("WebSocket 服务启动成功，地址: {}:{}", BIND_HOST, BIND_PORT);
    }

    /**
     * 初始化 HTTP 服务
     */
    private void initializeHttpServer() throws Exception {
        logger.info("初始化 HTTP 服务...");
        NetServer httpServer = new NettyServer(
                BIND_HTTP_HOST,
                BIND_HTTP_PORT,
                new InitalizerHttpServerHandler(new HttpInnerListener())
        );
        httpServer.start();
        logger.info("HTTP 服务启动成功，地址: {}:{}", BIND_HTTP_HOST, BIND_HTTP_PORT);
    }

    /**
     * 初始化中控服务（可选功能）
     */
    private void initializeCenterService() {
        if (StringUtils.isEmpty(CENTER_HOST) || CENTER_PORT <= 0) {
            logger.warn("中控服务地址未配置，跳过初始化...");
            return;
        }

        logger.info("初始化中控服务...");
        try {
            NetClient centerClient = new NettyClient(
                    CENTER_HOST,
                    CENTER_PORT,
                    new WebSocketServerBinaryAdapterHandler(
                            "/ws",
                            new InitalizerProtocolHandler(new CenterClientListener())
                    )
            );
            centerClient.enableReconnect(scheduleExec, TimeUnit.SECONDS.toMillis(5));
            centerClient.setName("CenterClient");
            centerClient.start();
            logger.info("中控服务连接成功，地址: {}:{}", CENTER_HOST, CENTER_PORT);
        } catch (Exception e) {
            logger.error("中控服务连接失败: {}", e.getMessage(), e);
        }
    }

}
