package com.incubator.game.room;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.util.RandomUtil;
import com.incubator.game.player.Player;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;
import org.slf4j.Logger;

import java.util.*;
import java.util.concurrent.*;

/**
 * 房间 管理类
 */
public class RoomManager {

    protected Logger logger = Log4jUtil.getLogger(getClass());

    private static RoomManager instance;
    /** 房间对象池 **/
    public Queue<Room> roomPool;
    /** 最大空闲房间数量 **/
    private int maxIdleRooms;
    /** 房间清理线程 **/
    private ScheduledExecutorService cleaner;

    /** 房间缓存 [key:房间类型, value:房间实例] **/
    public Map<Integer, Room> roomMap = new ConcurrentHashMap<>();

    public synchronized static RoomManager getInstance() {
        if (instance == null) {
            instance = new RoomManager();
        }
        return instance;
    }

    /**
     * 初始化
     *
     * @param maxIdleRooms 最大空闲房间数量
     */
    public void init(int maxIdleRooms) {
        logger.info("初始化房间服务...");
        this.roomPool = new ConcurrentLinkedQueue<>();
        this.maxIdleRooms = maxIdleRooms;
        this.cleaner = Executors.newSingleThreadScheduledExecutor();
        // 定期清理空闲房间
        this.cleaner.scheduleAtFixedRate(this::cleanIdleRooms, 10, 30, TimeUnit.SECONDS);
    }

    /**
     * 获取房间对象
     *
     * @param roomType
     * @return
     */
    public Room acquireRoom(int roomType) {
        Room room = this.roomPool.poll();
        if (room == null) {
            // 创建指定类型的房间
            room = RoomFactory.createRoom(roomType);
        }
        return room;
    }

    /**
     * 归还房间对象
     */
    public void releaseRoom(Room room) {
        if (room != null) {
            room.reset(); // 重置房间状态
            this.roomPool.offer(room); // 放回池中
        }
    }

    /**
     * 定期清理空闲房间
     */
    private void cleanIdleRooms() {
        while (this.roomPool.size() > maxIdleRooms) {
            Room room = this.roomPool.poll();
            if (room != null) {
                // 销毁房间，释放资源
                room.destroy();
            }
        }
    }

    /**
     * 关闭对象池
     */
    public void shutdown() {
        this.cleaner.shutdownNow();
        this.roomPool.forEach(Room::destroy);
        this.roomPool.clear();
    }

    public int getAvailableRooms() {
        return this.roomPool.size();
    }

    /**
     * 每日刷新
     */
    public void refreshNewDay() {

    }

    /**
     * 生成唯一6位数房间号
     *
     * @return
     */
    public int randomRoomId() {
        int roomNumber;
        // 循环生成，直到找到一个唯一的房间号
        do {
            roomNumber = RandomUtil.randomRoomNumber();
        } while (this.roomMap.containsKey(roomNumber));
        return roomNumber;
    }

    /**
     * 获取房间
     *
     * @param roomId
     * @return
     */
    public Room getRoom(int roomId) {
        return this.roomMap.getOrDefault(roomId, null);
    }

    /**
     * 获取玩家房间
     *
     * @param playerId
     * @return
     */
    public Room getRoomByPlayerId(String playerId) {
        for (Room room : this.roomMap.values()) {
            if (room != null && room.data.playerIds != null && room.data.playerIds.containsKey(playerId)) {
                return room;
            }
        }
        return null;
    }

    /**
     * 创建房间
     *
     * @param player
     * @param roomType
     * @return
     */
    public Room creatRoom(Player player, int roomType) {
        // 从对象池获取房间
        Room room = this.acquireRoom(roomType);
        int roomId = randomRoomId();
        // 初始化房间
        room.init(player, roomId);
        // 加入管理映射
        this.roomMap.put(room.data.roomId, room);
        return room;
    }

    /**
     * 加入房间
     *
     * @param room
     * @param player
     * @return
     */
    public Room joinRoom(Room room, Player player) {
        if (!room.data.playerIds.containsKey(player.getId())) {
            room.data.playerIds.put(player.getId(), player);
        }
        int index = 0;
        for (Player tmpPlayer : room.data.playerIds.values()) {
            if (tmpPlayer != null) {
                tmpPlayer.data.pos = index;
                room.data.playerMap.put(index, tmpPlayer);
                index++;
            }
        }

        // 推送房间其他3个玩家
        for (Player tmPlayer : room.data.playerMap.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = ProtoUtil.roomToMessage(room, tmPlayer, null);
                tmPlayer.receive(CommonProto.Cmd.RoomPlayerJoin_VALUE, data);
            }
        }

        return room;
    }
}
