package com.incubator.game.util;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 经典掼蛋 工具类
 */
public final class JDGDUtils {

    /**
     * 掼蛋对应的数值
     * 0x02-0x0e 方片（♦️） 2(2) 3(3) 4(4) 5(5) 6(6) 7(7) 8(8) 9(9) 10(10) J(11) Q(12) K(13) A(14)
     * 0x12-0x1e 梅花（♣️） 2(18) 3(19) 4(20) 5(21) 6(22) 7(23) 8(24) 9(25) 10(26) J(27) Q(28) K(29) A(30)
     * 0x22-0x2e 红桃（♥️） 2(34) 3(35) 4(36) 5(37) 6(38) 7(39) 8(40) 9(41) 10(42) J(43) Q(44) K(45) A(46)
     * 0x32-0x3e 黑桃（♠️） 2(50) 3(51) 4(52) 5(53) 6(54) 7(55) 8(56) 9(57) 10(58) J(59) Q(60) K(61) A(62)
     * 0xef      小王(239)
     * 0xff      大王(255)
     */
    public static Integer[] values = {
            0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e,
            0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e,
            0x22, 0x23, 0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2a, 0x2b, 0x2c, 0x2d, 0x2e,
            0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x3a, 0x3b, 0x3c, 0x3d, 0x3e,
            0xef,
            0xff
    };

    /** 级数配置 2、3、4、5、6、7、8、9、10、J、Q、K、A **/
    public static Integer[] levelPoint = {
            2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14
    };

    // 牌型枚举
    public enum CardType {
        SINGLE,              // 单张
        PAIR,                // 对子
        PAIR_SEQUENCE,       // 3连对(3个连续对子)
        TRIPLE,              // 三不带
        TRIPLE_WITH_TWO,     // 三带二
        TRIPLE_SEQUENCE,     // 三顺(二个连续三张)
        STRAIGHT,            // 顺子(5张牌)
        BOMB,                // 通用炸弹
        FOUR_BOMB,           // 四炸弹
        FIVE_BOMB,           // 五炸弹
        SIX_BOMB,            // 六炸弹
        SEVEN_BOMB,          // 七炸弹
        EIGHT_BOMB,          // 八炸弹
        FOUR_KINGS,          // 四王炸
        ROYAL_FLUSH,         // 同花顺（可选）
        INVALID              // 无效牌型
    }

    /**
     * 是否是炸弹
     *
     * @param type
     * @return
     */
    public static boolean isBomb(CardType type) {
        return type == CardType.BOMB
                || type == CardType.FOUR_BOMB
                || type == CardType.FIVE_BOMB
                || type == CardType.SIX_BOMB
                || type == CardType.SEVEN_BOMB
                || type == CardType.EIGHT_BOMB
                || type == CardType.FOUR_KINGS
                || type == CardType.ROYAL_FLUSH;
    }

    /**
     * 比较两组牌的大小（仅在牌型相同时调用）
     *
     * @param previousCards 上一手牌
     * @param disCardList 当前玩家出的牌
     * @param curLevel 当前级数
     * @param disCardsType 出牌牌型
     * @return 正数表示当前牌较大，负数表示上一手牌较大，0表示相等
     */
    public static int compareCardStrength(int[] previousCards, int[] disCardList, int curLevel, CardType disCardsType) {
        List<Integer> previousPoints = Arrays.stream(previousCards)
                .map(card -> getCardPoint(card, curLevel))
                .sorted()
                .boxed()
                .collect(Collectors.toList());

        List<Integer> currentPoints = Arrays.stream(disCardList)
                .map(card -> getCardPoint(card, curLevel))
                .sorted()
                .boxed()
                .collect(Collectors.toList());

        switch (disCardsType) {
            case SINGLE:
            case PAIR:
            case TRIPLE:
            case STRAIGHT:
                // 单张、对子、三不带、顺子直接比较最大点数
                return Integer.compare(
                        currentPoints.get(currentPoints.size() - 1),
                        previousPoints.get(previousPoints.size() - 1)
                );

            case TRIPLE_WITH_TWO:
                // 三带二：优先比较三张部分，再比较两张部分
                int currentTriple = getMaxFrequencyCard(currentPoints, 3);
                int previousTriple = getMaxFrequencyCard(previousPoints, 3);

                if (currentTriple != previousTriple) {
                    return Integer.compare(currentTriple, previousTriple);
                }

                // 如果三张部分相等，比较附带的两张部分
                List<Integer> currentPair = currentPoints.stream()
                        .filter(point -> point != currentTriple)
                        .sorted(Comparator.reverseOrder())
                        .collect(Collectors.toList());
                List<Integer> previousPair = previousPoints.stream()
                        .filter(point -> point != previousTriple)
                        .sorted(Comparator.reverseOrder())
                        .collect(Collectors.toList());

                for (int i = 0; i < currentPair.size(); i++) {
                    int cmp = Integer.compare(currentPair.get(i), previousPair.get(i));
                    if (cmp != 0) {
                        return cmp;
                    }
                }
                return 0;

            case PAIR_SEQUENCE:
            case TRIPLE_SEQUENCE:
                // 连对、三顺：逐一比较点数
                for (int i = 0; i < currentPoints.size(); i++) {
                    int cmp = Integer.compare(currentPoints.get(i), previousPoints.get(i));
                    if (cmp != 0) {
                        return cmp;
                    }
                }
                return 0;

            case BOMB:
            case FOUR_BOMB:
            case FIVE_BOMB:
            case SIX_BOMB:
            case SEVEN_BOMB:
            case EIGHT_BOMB:
            case FOUR_KINGS:
                // 炸弹：按炸弹的牌型权重比较
                int currentWeight = getCardTypeWeight(determineCardType(disCardList, curLevel));
                int previousWeight = getCardTypeWeight(determineCardType(previousCards, curLevel));

                if (currentWeight != previousWeight) {
                    return Integer.compare(currentWeight, previousWeight);
                }
                // 若权重相等，比较炸弹点数
                return Integer.compare(
                        currentPoints.get(currentPoints.size() - 1),
                        previousPoints.get(previousPoints.size() - 1)
                );

            default:
                // 对于未定义类型的比较，按最大点数处理
                return Integer.compare(
                        currentPoints.get(currentPoints.size() - 1),
                        previousPoints.get(previousPoints.size() - 1)
                );
        }
    }

    /**
     * 获取指定频率的最大点数
     *
     * @param points 点数列表
     * @param frequency 目标频率（如 3 表示三张）
     * @return 满足频率的最大点数
     */
    private static int getMaxFrequencyCard(List<Integer> points, int frequency) {
        return points.stream()
                .filter(p -> Collections.frequency(points, p) == frequency)
                .max(Integer::compareTo)
                .orElse(0);
    }

    /**
     * 获取牌点，考虑级牌影响
     * @param card 牌的值
     * @param curLevel 当前级数
     * @return 牌点
     */
    public static int getCardPoint(int card, int curLevel) {
        if (card == 0xEF) return 16; // 小王
        if (card == 0xFF) return 17; // 大王

        int point = card & 0xF; // 其他牌点
        if (point == curLevel) {
            return 15; // 级牌
        }
        return point;
    }

    /**
     * 获取牌型的权重值
     * @param cardType 牌型
     * @return 权重值
     */
    public static int getCardTypeWeight(CardType cardType) {
        switch (cardType) {
            case FOUR_KINGS: return 7; // 4个王 > ...
            case EIGHT_BOMB: return 6; // 8炸弹 > ...
            case SEVEN_BOMB: return 5; // 7炸弹 > ...
            case SIX_BOMB: return 4;   // 6炸弹 > ...
            case ROYAL_FLUSH: return 3;// 同花顺 > ...
            case FIVE_BOMB: return 2;  // 5炸弹 > ...
            case FOUR_BOMB: return 1;  // 4炸弹 > ...
            default: return 0; // 其他牌型
        }
    }

    /**
     * 判断牌型
     * @param disCardList 玩家出的牌（int数组）
     * @param curLevel 当前级数
     * @return 牌型枚举值
     */
    public static CardType determineCardType(int[] disCardList, int curLevel) {
        if (disCardList == null || disCardList.length == 0) {
            return CardType.INVALID;
        }

        // 提取牌点
        List<Integer> points = Arrays.stream(disCardList)
                .map(card -> getCardPoint(card, curLevel))
                .boxed()
                .collect(Collectors.toList());

        // 统计牌点频率
        Map<Integer, Long> pointFrequency = points.stream()
                .collect(Collectors.groupingBy(p -> p, Collectors.counting()));

        // 排序后的牌点集合
        List<Integer> sortedPoints = points.stream()
                .distinct()
                .sorted()
                .collect(Collectors.toList());

        // 判断不同牌型
        int cardCount = disCardList.length;
        int uniqueCount = pointFrequency.size();

        // 1. 单张
        if (cardCount == 1) {
            return CardType.SINGLE;
        }

        // 2. 对子
        if (cardCount == 2 && uniqueCount == 1) {
            return CardType.PAIR;
        }

        // 3. 四王炸（大小王各两张）
        if (cardCount == 4 && pointFrequency.size() == 2
                && points.contains(16) && points.contains(17)
                && pointFrequency.get(16) == 2L && pointFrequency.get(17) == 2L) {
            return CardType.FOUR_KINGS;
        }

        // 4. 特定炸弹类型（四炸弹到八炸弹）
        if (uniqueCount == 1 && cardCount >= 4 && cardCount <= 8) {
            switch (cardCount) {
                case 4: return CardType.FOUR_BOMB;
                case 5: return CardType.FIVE_BOMB;
                case 6: return CardType.SIX_BOMB;
                case 7: return CardType.SEVEN_BOMB;
                case 8: return CardType.EIGHT_BOMB;
                default: break;
            }
        }

        // 5. 通用炸弹（四张及以上相同牌点）
        if (uniqueCount == 1 && cardCount >= 4) {
            return CardType.BOMB;
        }

        // 6. 顺子(5张牌)
        if (cardCount == 5 && uniqueCount == cardCount && isStraight(sortedPoints, curLevel)) {
            return CardType.STRAIGHT;
        }

        // 7. 3连对(3个连续对子)
        if (cardCount == 6 && uniqueCount * 2 == cardCount && isPairSequence(pointFrequency, curLevel)) {
            return CardType.PAIR_SEQUENCE;
        }

        // 8. 三不带
        if (cardCount == 3 && uniqueCount == 1) {
            return CardType.TRIPLE;
        }

        // 9. 三带二
        if (cardCount == 5 && uniqueCount == 2 && pointFrequency.containsValue(3L) && pointFrequency.containsValue(2L)) {
            return CardType.TRIPLE_WITH_TWO;
        }

        // 10. 三顺(二个连续三张)
        if (cardCount == 6 && uniqueCount == cardCount / 3 && isTripleSequence(pointFrequency, curLevel)) {
            return CardType.TRIPLE_SEQUENCE;
        }

        // 11. 同花顺（可选规则）
        if (isRoyalFlush(disCardList, curLevel)) {
            return CardType.ROYAL_FLUSH;
        }

        return CardType.INVALID; // 非法牌型
    }

    // 判断是否为顺子（点数连续，且不包含 2 和大小王）
    private static boolean isStraight(List<Integer> points, int currentLevel) {
        for (int i = 0; i < points.size() - 1; i++) {
            if (points.get(i + 1) - points.get(i) != 1 || points.get(i) >= 15) { // 不允许 2、小王、大王
                return false;
            }
        }
        return true;
    }

    // 判断是否为连对
    private static boolean isPairSequence(Map<Integer, Long> pointFrequency, int currentLevel) {
        List<Integer> sortedKeys = pointFrequency.keySet().stream().sorted().collect(Collectors.toList());
        for (int i = 0; i < sortedKeys.size() - 1; i++) {
            if (sortedKeys.get(i + 1) - sortedKeys.get(i) != 1 || pointFrequency.get(sortedKeys.get(i)) != 2) {
                return false;
            }
        }
        return true;
    }

    // 判断是否为三连顺
    private static boolean isTripleSequence(Map<Integer, Long> pointFrequency, int currentLevel) {
        List<Integer> triples = pointFrequency.entrySet().stream()
                .filter(entry -> entry.getValue() == 3L)
                .map(Map.Entry::getKey)
                .sorted()
                .collect(Collectors.toList());
        for (int i = 0; i < triples.size() - 1; i++) {
            if (triples.get(i + 1) - triples.get(i) != 1 || triples.get(i) >= 15) { // 不允许 2、小王、大王
                return false;
            }
        }
        return triples.size() >= 2;
    }

    // 判断是否为同花顺（可选规则）
    private static boolean isRoyalFlush(int[] cards, int currentLevel) {
        return Arrays.stream(cards).map(card -> card & 0xF0).distinct().count() == 1
                && isStraight(Arrays.stream(cards).map(card -> getCardPoint(card, currentLevel)).sorted().boxed().collect(Collectors.toList()), currentLevel);
    }

    // 测试代码
    public static void main(String[] args) {
        int[] cards1 = {0x08, 0x04, 0x05, 0x06, 0x07}; // 顺子
        int[] cards2 = {0x33, 0x13, 0x23, 0x03};       // 四炸弹
        int[] cards8 = {0xEF, 0xEF, 0xFF, 0xFF};             // 四王炸
        int[] cards9 = {0x03, 0x13, 0x23, 0x33};             // 四炸弹
        int[] cards10 = {0x04, 0x14, 0x24, 0x34, 0x44};      // 五炸弹
        int[] cards11 = {0x05, 0x15, 0x25, 0x35, 0x45, 0x55}; // 六炸弹
        int[] cards12 = {0x06, 0x16, 0x26, 0x36, 0x46, 0x56, 0x66}; // 七炸弹
        int[] cards13 = {0x07, 0x17, 0x27, 0x37, 0x47, 0x57, 0x67, 0x77}; // 八炸弹

        System.out.println(determineCardType(cards1, 3)); // STRAIGHT
        System.out.println(determineCardType(cards2, 3)); // BOMB
        System.out.println(determineCardType(cards8, 3)); // FOUR_KINGS
        System.out.println(determineCardType(cards9, 3)); // FOUR_BOMB
        System.out.println(determineCardType(cards10, 3)); // FIVE_BOMB
        System.out.println(determineCardType(cards11, 3)); // SIX_BOMB
        System.out.println(determineCardType(cards12, 3)); // SEVEN_BOMB
        System.out.println(determineCardType(cards13, 3)); // EIGHT_BOMB
    }
}